/*
 * $Header: /cvsroot/d2kbasic/basic/src/ncsa/d2k/modules/core/transform/graph/JungToPrefuse.java,v 1.4 2006/08/02 15:05:47 clutter Exp $
 *
 * ===================================================================
 *
 * D2K-Workflow
 * Copyright (c) 1997,2006 THE BOARD OF TRUSTEES OF THE UNIVERSITY OF
 * ILLINOIS. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License v2.0
 * as published by the Free Software Foundation and with the required
 * interpretation regarding derivative works as described below.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License v2.0 for more details.
 *
 * This program and the accompanying materials are made available
 * under the terms of the GNU General Public License v2.0 (GPL v2.0)
 * which accompanies this distribution and is available at
 * http://www.gnu.org/copyleft/gpl.html (or via mail from the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.), with the special and mandatory
 * interpretation that software only using the documented public
 * Application Program Interfaces (APIs) of D2K-Workflow are not
 * considered derivative works under the terms of the GPL v2.0.
 * Specifically, software only calling the D2K-Workflow Itinerary
 * execution and workflow module APIs are not derivative works.
 * Further, the incorporation of published APIs of other
 * independently developed components into D2K Workflow code
 * allowing it to use those separately developed components does not
 * make those components a derivative work of D2K-Workflow.
 * (Examples of such independently developed components include for
 * example, external databases or metadata and provenance stores).
 *
 * Note: A non-GPL commercially licensed version of contributions
 * from the UNIVERSITY OF ILLINOIS may be available from the
 * designated commercial licensee RiverGlass, Inc. located at
 * (www.riverglassinc.com)
 * ===================================================================
 *
 */
package ncsa.d2k.modules.core.util;

import ncsa.d2k.core.modules.ComputeModule;
import ncsa.d2k.core.modules.PropertyDescription;
import ncsa.d2k.modules.core.discovery.ruleassociation.ItemSets;
//import ncsa.d2k.core.modules.*;
import java.io.*;
import java.nio.*;
import java.util.*;

public class ExecExternal extends ComputeModule {

	protected String executableProgram = "not set";
	
	public String getExecutableProgram() {
		return this.executableProgram;
	}
	public void setExecutableProgram( String s ) {
		this.executableProgram = s;
	}
	protected String arguments = "";
	public String getArguments() {
		return this.arguments;
	}
	public void setArguments( String s ) {
		this.arguments = s;
	}
	
	protected String outputFile = "";
	public String getOutputFile() {
		return this.outputFile;
	}
	public void setOutputFile( String s ) {
		this.outputFile = s;
	}
	
	
   //~ Methods *****************************************************************
	public PropertyDescription[] getPropertiesDescriptions() {
		PropertyDescription[] pds = new PropertyDescription[4];
		pds[0] =
			new PropertyDescription(
				"executableProgram",
				"Executable",
				"The program to execute (full path)");
		pds[1] =
			new PropertyDescription(
				"arguments",
				"Arguments",
				"The arguments to the program.");
		pds[2] =
			new PropertyDescription(
				"outputFile",
				"Output File",
				"The primary output of the program.");
		pds[3] = 
			new PropertyDescription(
				"debug",
				"Generate Verbose Output",
				"If this property is true, the module will write verbose status information to the console.");
		return pds;
	}
	
	protected boolean debug = false;
	public void setDebug(boolean yy) {
		this.debug = yy;
	}

	public boolean getDebug() {
		return this.debug;
	}
	
	 /**
	    * Called by the D2K Infrastructure before the itinerary begins to execute.
	    *
	    * <p>override in subclasses to fill in the command
	    * from arguments and inputs.</p>
	    */
	   public void beginExecution() {
	       }
	   /**
	    * Called by the D2K Infrastructure before the itinerary begins to execute.
	    *
	    * <p>override in subclasses to fill in the command
	    * from arguments and inputs.</p>
	    */
	   public void endExecution() {
	       }
	   
	   /* These may be used in subclasses to implement the details of
	    * a particular program invocation and result. */
	protected String theCommand ="";
	protected String outText = null;
	protected int retValue = 0;

	/* override this in sub classes */
	protected String setup() throws Exception {
		return executableProgram+" "+arguments;
	}
	/* override this in sub classes */
	protected void finishIt() throws Exception {
		
	}
	
	
   /**
    * Performs the main work of the module.
    * 
    * <p>'setup' is called to construct the command to be executed. This will be
    * overridden by a subclass to create the application specific string.
    * 
    * <p>The String is executed using Runtime.exec(). The exit value is stored
    * in the variable retValue. The standard output and error are saved in the
    * variable outText.
    * 
    * <p>'finishIt' is called to finish the processing. Typically this will be 
    * overridden by a subclass to push out results or raise an exception if the
    * program failed.
    *
    * @throws Exception if a problem occurs while performing the work of the
    *                   module
    */
   public void doit() throws Exception {
	  setup();
	  
	  if (debug) {
	    	  System.out.println("execute it: "+theCommand);
	      }
	  Process pp = Runtime.getRuntime().exec(theCommand);
	  BufferedInputStream bis1 = new BufferedInputStream(pp.getErrorStream());
	  BufferedInputStream bis2 = new BufferedInputStream(pp.getInputStream());
	  retValue = pp.waitFor();
	  StringBuffer sb = new StringBuffer();
	  while (bis1.available() > 0) {
	    	byte bbx[] = new byte[bis1.available()];
	    	int bx = bis1.read(bbx);  
	    	String s2 = new String(bbx);
	    	sb.append(s2);
	  }
	  while (bis2.available() > 0) {
	    	byte bbx[] = new byte[bis2.available()];
	    	int bx = bis2.read(bbx);  
	    	String s2 = new String(bbx);
	    	sb.append(s2);
	  }
	  outText = sb.toString();
	  if (debug) {
		System.out.println(outText);
	    System.out.println("exit value: "+retValue); 	 
	  }
	    
      finishIt();   
      
   } // end method doit

   /**
    * Returns a description of the input at the specified index.
    *
    * @param  i Index of the input for which a description should be returned.
    *
    * @return <code>String</code> describing the input at the specified index.
    */
   public String getInputInfo(int i) {

	  switch (i) {
	  case 0:
		  return "Input File Name";
	  default:
			  return null;
	  }
      
   }

   /**
    * Returns the name of the input at the specified index.
    *
    * @param  i Index of the input for which a name should be returned.
    *
    * @return <code>String</code> containing the name of the input at the
    *         specified index.
    */
   public String getInputName(int i) {

	   switch (i) {
		  case 0:
         return "Input File";
		  default:
			  return null;
	  }
      
   }

   /**
    * Returns an array of <code>String</code> objects each containing the fully
    * qualified Java data type of the input at the corresponding index.
    *
    * @return An array of <code>String</code> objects each containing the fully
    *         qualified Java data type of the input at the corresponding index.
    */
   public String[] getInputTypes() { 
	   return new String[] { 
			   "java.lang.String"};
	   }

   /**
    * Describes the purpose of the module.
    *
    * @return <code>String</code> describing the purpose of the module.
    */
   public String getModuleInfo() {
      String s =
         "<p>Overview: This class is a generic wrapper for executing" +
	         "a (Unix) program.  The program is executed via the Runtime.exec()" +
	         "call." +
	         "<p>The simple case has an excutable program" +
	         "(a path) and arguments." +
	         "<p>In order to handle the variations in command lines, the" +
	         "'setup' and 'finishIt' methods should be overwritten. Typically," +
	         " setup is used by the subclass to construct the invocation in " +
	         "the String 'theCommand', while 'finishIt' is used to push out" +
	         "the name of one or more output files.";
      

      return s;
   }

   /**
    * Returns the name of the module that is appropriate for end-user
    * consumption.
    *
    * @return The name of the module.
    */
   public String getModuleName() { return "Execute External Program"; }

   /**
    * Returns a description of the output at the specified index.
    *
    * @param  i Index of the output for which a description should be returned.
    *
    * @return <code>String</code> describing the output at the specified index.
    */
   public String getOutputInfo(int i) {

	   	switch (i) {
		  case 0:
			  return "Output File";
		  default:
			  return null;
	   	}
   }

   /**
    * Returns the name of the output at the specified index.
    *
    * @param  i Index of the output for which a description should be returned.
    *
    * @return <code>String</code> containing the name of the output at the
    *         specified index.
    */
   public String getOutputName(int i) {

	 	switch (i) {
		  case 0:
			  return "Output File";
		  default:
			  return null;
	   	}

   }

   /**
    * Returns an array of <code>String</code> objects each containing the fully
    * qualified Java data type of the output at the corresponding index.
    *
    * @return An array of <code>String</code> objects each containing the fully
    *         qualified Java data type of the output at the corresponding index.
    */
   public String[] getOutputTypes() { return new String[] {
                                                "java.lang.String"
                                             }; }
} 
