package ncsa.d2k.modules.core.util;

public class ExecSoftMean extends ExecExternal {
    private String outfile = null;

    /**
        * Returns a description of the input at the specified index.
        *
        * @param  i Index of the input for which a description should be returned.
        *
        * @return <code>String</code> describing the input at the specified index.
        */
    public String getInputInfo(int i) {

        switch (i) {

        case 0:
            return "Input Warp 1";

        case 2:
            return "Input Warp 2";

        case 3:
            return "Input Warp 3";

        case 4:
            return "Input Warp 4";

        default:
            return null;
        }
    }

    /**
      * Describes the purpose of the module.
      *
      * @return <code>String</code> describing the purpose of the module.
      */
    public String getModuleInfo() {
        String s =
            "<p>Overview: Calls the ARI 'softmean' program with 4 input " +
            "files.";


        return s;
    }

    /**
     * Returns the name of the module that is appropriate for end-user
     * consumption.
     *
     * @return The name of the module.
     */
    public String getModuleName() {
        return "Execute softmean Program";
    }


    /**
     * Returns the name of the input at the specified index.
     *
     * @param  i Index of the input for which a name should be returned.
     *
     * @return <code>String</code> containing the name of the input at the
     *         specified index.
     */
    public String getInputName(int i) {

        switch (i) {

        case 0:
            return "Warp File";

        case 1:
            return "Warp File";

        case 2:
            return "Warp File";

        case 3:
            return "Warp File";

        default:
            return null;
        }

    }

    /**
     * Returns an array of <code>String</code> objects each containing the fully
     * qualified Java data type of the input at the corresponding index.
     *
     * @return An array of <code>String</code> objects each containing the fully
     *         qualified Java data type of the input at the corresponding index.
     */
    public String[] getInputTypes() {
        return new String[] {
                "java.lang.String", "java.lang.String", "java.lang.String",
                "java.lang.String"
            };
    }

    protected String setup() throws Exception {

        String in1 = (String) this.pullInput(0);
        String in2 = (String) this.pullInput(1);
        String in3 = (String) this.pullInput(2);
        String in4 = (String) this.pullInput(3);
        outfile = outputFile;

        theCommand = executableProgram + " " + outfile + " " + arguments + " " +
            in1 + " " + in2 + " " + in3 + " " + in4 + " ";

        return theCommand;
    }

    public void finishIt() throws Exception {

        if (retValue != 0) {
            throw new ExecExternalException(outText);
        }

        if (debug) {
            System.out.println("pushOutput " + outfile);
        }

        pushOutput(outfile, 0);
    }

}
