
package ncsa.d2k.modules.core.util;

import java.util.*;


public class ExecWarpAlign extends ExecExternal {
    private String outfile = null;

    /**
       * Returns a description of the input at the specified index.
       *
       * @param  i Index of the input for which a description should be returned.
       *
       * @return <code>String</code> describing the input at the specified index.
       */
    public String getInputInfo(int i) {

        switch (i) {

        case 0:
            return "Input Image File Name";

        case 2:
            return "Input Reference File Name";

        default:
            return null;
        }
    }

    /**
      * Describes the purpose of the module.
      *
      * @return <code>String</code> describing the purpose of the module.
      */
    public String getModuleInfo() {
        String s = "<p>Overview: Call the AIR 'align_warp' program.";


        return s;
    }

    /**
     * Returns the name of the module that is appropriate for end-user
     * consumption.
     *
     * @return The name of the module.
     */
    public String getModuleName() {
        return "Execute align_warp Program";
    }


    /**
     * Returns the name of the input at the specified index.
     *
     * @param  i Index of the input for which a name should be returned.
     *
     * @return <code>String</code> containing the name of the input at the
     *         specified index.
     */
    public String getInputName(int i) {

        switch (i) {

        case 0:
            return "Image File";

        case 1:
            return "Reference File";

        default:
            return null;
        }

    }

    /**
     * Returns an array of <code>String</code> objects each containing the fully
     * qualified Java data type of the input at the corresponding index.
     *
     * @return An array of <code>String</code> objects each containing the fully
     *         qualified Java data type of the input at the corresponding index.
     */
    public String[] getInputTypes() {
        return new String[] { "java.lang.String", "java.lang.String" };
    }

    protected String setup() throws Exception {
        theCommand = executableProgram + " " + (String) this.pullInput(0) +
            " " + (String) this.pullInput(1) + " " + (outfile = outputFile) +
            " " + arguments;

        return theCommand;
    }

    public void finishIt() throws Exception {

        if (retValue != 0) {
            throw new ExecExternalException(outText);
        }

        if (debug) {
            System.out.println("pushOutput " + outfile);
        }

        pushOutput(outfile, 0);
    }
}
